"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.checkToken = exports.withLogger = void 0;
const crypto_1 = require("crypto");
const jsonwebtoken_1 = __importDefault(require("jsonwebtoken"));
const kv_logger_1 = require("kv-logger");
const lodash_1 = __importDefault(require("lodash"));
const moment_1 = __importDefault(require("moment"));
const sequelize_1 = require("sequelize");
const user_tokens_1 = require("../models/user_tokens");
const users_1 = require("../models/users");
const app_error_1 = require("./app-error");
const config_1 = require("./config");
const security_1 = require("./utils/security");
/**
 * bind logger to request
 */
function withLogger(req, res, next) {
    const { method, path, headers } = req;
    const requestId = headers['x-request-id'] || (0, crypto_1.randomUUID)();
    req.logger = kv_logger_1.logger.bindContext({
        path,
        method,
        requestId,
    });
    res.header('X-Request-Id', requestId);
    next();
}
exports.withLogger = withLogger;
async function checkAuthToken(authToken) {
    const objToken = (0, security_1.parseToken)(authToken);
    const users = await users_1.Users.findOne({
        where: { identical: objToken.identical },
    });
    if (lodash_1.default.isEmpty(users)) {
        throw new app_error_1.Unauthorized();
    }
    const tokenInfo = await user_tokens_1.UserTokens.findOne({
        where: {
            tokens: authToken,
            uid: users.id,
            expires_at: {
                [sequelize_1.Op.gt]: (0, moment_1.default)().format('YYYY-MM-DD HH:mm:ss'),
            },
        },
    });
    if (lodash_1.default.isEmpty(tokenInfo)) {
        throw new app_error_1.Unauthorized();
    }
    return users;
}
async function checkAccessToken(accessToken) {
    if (lodash_1.default.isEmpty(accessToken)) {
        throw new app_error_1.Unauthorized();
    }
    let authData;
    try {
        authData = jsonwebtoken_1.default.verify(accessToken, config_1.config.jwt.tokenSecret);
    }
    catch (e) {
        throw new app_error_1.Unauthorized();
    }
    const { uid, hash } = authData;
    if (uid <= 0) {
        throw new app_error_1.Unauthorized();
    }
    const users = await users_1.Users.findOne({
        where: { id: uid },
    });
    if (lodash_1.default.isEmpty(users)) {
        throw new app_error_1.Unauthorized();
    }
    if (hash !== (0, security_1.md5)(users.get('ack_code'))) {
        throw new app_error_1.Unauthorized();
    }
    return users;
}
/**
 * check user token and bind user to request
 */
function checkToken(req, res, next) {
    // get token and type
    let authType = 1;
    let authToken = '';
    const authArr = lodash_1.default.split(req.get('Authorization'), ' ');
    if (authArr[0] === 'Bearer') {
        [, authToken] = authArr; // Bearer
        if (authToken && authToken.length > 64) {
            authType = 2;
        }
        else {
            authType = 1;
        }
    }
    else if (authArr[0] === 'Basic') {
        authType = 2;
        const b = Buffer.from(authArr[1], 'base64');
        const user = lodash_1.default.split(b.toString(), ':');
        [, authToken] = user;
    }
    // do check token
    let checkTokenResult;
    if (authToken && authType === 1) {
        checkTokenResult = checkAuthToken(authToken);
    }
    else if (authToken && authType === 2) {
        checkTokenResult = checkAccessToken(authToken);
    }
    else {
        res.send(new app_error_1.Unauthorized(`Auth type not supported.`));
        return;
    }
    checkTokenResult
        .then((users) => {
        req.users = users;
        next();
    })
        .catch((e) => {
        if (e instanceof app_error_1.AppError) {
            res.status(e.status || 404).send(e.message);
        }
        else {
            next(e);
        }
    });
}
exports.checkToken = checkToken;
