"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.appManager = void 0;
const lodash_1 = __importDefault(require("lodash"));
const sequelize_1 = require("sequelize");
const apps_1 = require("../../models/apps");
const collaborators_1 = require("../../models/collaborators");
const deployments_1 = require("../../models/deployments");
const users_1 = require("../../models/users");
const app_error_1 = require("../app-error");
const const_1 = require("../const");
const connections_1 = require("../utils/connections");
const security_1 = require("../utils/security");
class AppManager {
    findAppByName(uid, appName) {
        return apps_1.Apps.findOne({ where: { name: appName, uid } });
    }
    addApp(uid, appName, os, platform, identical) {
        return connections_1.sequelize.transaction((t) => {
            return apps_1.Apps.create({
                name: appName,
                uid,
                os,
                platform,
            }, {
                transaction: t,
            }).then((apps) => {
                const appId = apps.id;
                const deployments = [];
                let deploymentKey = (0, security_1.randToken)(28) + identical;
                deployments.push({
                    appid: appId,
                    name: const_1.PRODUCTION,
                    last_deployment_version_id: 0,
                    label_id: 0,
                    deployment_key: deploymentKey,
                });
                deploymentKey = (0, security_1.randToken)(28) + identical;
                deployments.push({
                    appid: appId,
                    name: const_1.STAGING,
                    last_deployment_version_id: 0,
                    label_id: 0,
                    deployment_key: deploymentKey,
                });
                return Promise.all([
                    collaborators_1.Collaborators.create({ appid: appId, uid, roles: 'Owner' }, { transaction: t }),
                    deployments_1.Deployments.bulkCreate(deployments, { transaction: t }),
                ]);
            });
        });
    }
    deleteApp(appId) {
        return connections_1.sequelize.transaction((t) => {
            return Promise.all([
                apps_1.Apps.destroy({ where: { id: appId }, transaction: t }),
                collaborators_1.Collaborators.destroy({ where: { appid: appId }, transaction: t }),
                deployments_1.Deployments.destroy({ where: { appid: appId }, transaction: t }),
            ]);
        });
    }
    modifyApp(appId, params) {
        return apps_1.Apps.update(params, { where: { id: appId } }).then(([affectedCount]) => {
            if (!lodash_1.default.gt(affectedCount, 0)) {
                throw new app_error_1.AppError('modify errors');
            }
            return affectedCount;
        });
    }
    transferApp(appId, fromUid, toUid) {
        return connections_1.sequelize.transaction((t) => {
            return Promise.all([
                apps_1.Apps.update({ uid: toUid }, { where: { id: appId }, transaction: t }),
                collaborators_1.Collaborators.destroy({ where: { appid: appId, uid: fromUid }, transaction: t }),
                collaborators_1.Collaborators.destroy({ where: { appid: appId, uid: toUid }, transaction: t }),
                collaborators_1.Collaborators.create({ appid: appId, uid: toUid, roles: 'Owner' }, { transaction: t }),
            ]);
        });
    }
    listApps(uid) {
        return collaborators_1.Collaborators.findAll({ where: { uid } })
            .then((data) => {
            if (lodash_1.default.isEmpty(data)) {
                return [];
            }
            const appIds = lodash_1.default.map(data, (v) => {
                return v.appid;
            });
            return apps_1.Apps.findAll({ where: { id: { [sequelize_1.Op.in]: appIds } } });
        })
            .then((appInfos) => {
            const rs = Promise.all(lodash_1.default.values(appInfos).map((v) => {
                return this.getAppDetailInfo(v, uid).then((info) => {
                    let os = '';
                    if (info.os === const_1.IOS) {
                        os = const_1.IOS_NAME;
                    }
                    else if (info.os === const_1.ANDROID) {
                        os = const_1.ANDROID_NAME;
                    }
                    else if (info.os === const_1.WINDOWS) {
                        os = const_1.WINDOWS_NAME;
                    }
                    let platform = '';
                    if (info.platform === const_1.REACT_NATIVE) {
                        platform = const_1.REACT_NATIVE_NAME;
                    }
                    else if (info.platform === const_1.CORDOVA) {
                        platform = const_1.CORDOVA_NAME;
                    }
                    return {
                        ...info,
                        os,
                        platform,
                    };
                });
            }));
            return rs;
        });
    }
    getAppDetailInfo(appInfo, currentUid) {
        const appId = appInfo.get('id');
        return Promise.all([
            deployments_1.Deployments.findAll({ where: { appid: appId } }),
            collaborators_1.Collaborators.findAll({ where: { appid: appId } }).then((collaboratorInfos) => {
                return collaboratorInfos.reduce((prev, collaborator) => {
                    return prev.then((allCol) => {
                        return users_1.Users.findOne({ where: { id: collaborator.get('uid') } }).then((u) => {
                            let isCurrentAccount = false;
                            if (lodash_1.default.eq(u.get('id'), currentUid)) {
                                isCurrentAccount = true;
                            }
                            allCol[u.get('email')] = {
                                permission: collaborator.get('roles'),
                                isCurrentAccount,
                            };
                            return allCol;
                        });
                    });
                }, Promise.resolve({}));
            }),
        ]).then(([deploymentInfos, collaborators]) => {
            return {
                collaborators,
                deployments: lodash_1.default.map(deploymentInfos, (item) => {
                    return lodash_1.default.get(item, 'name');
                }),
                os: appInfo.get('os'),
                platform: appInfo.get('platform'),
                name: appInfo.get('name'),
                id: appInfo.get('id'),
            };
        });
    }
}
exports.appManager = new AppManager();
