"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.clientManager = void 0;
const lodash_1 = __importDefault(require("lodash"));
const sequelize_1 = require("sequelize");
const deployments_1 = require("../../models/deployments");
const deployments_versions_1 = require("../../models/deployments_versions");
const log_report_deploy_1 = require("../../models/log_report_deploy");
const log_report_download_1 = require("../../models/log_report_download");
const packages_1 = require("../../models/packages");
const packages_diff_1 = require("../../models/packages_diff");
const packages_metrics_1 = require("../../models/packages_metrics");
const app_error_1 = require("../app-error");
const config_1 = require("../config");
const const_1 = require("../const");
const common_1 = require("../utils/common");
const connections_1 = require("../utils/connections");
const UPDATE_CHECK = 'UPDATE_CHECK';
const CHOSEN_MAN = 'CHOSEN_MAN';
const EXPIRED = 600;
class ClientManager {
    getUpdateCheckCacheKey(deploymentKey, appVersion, label, packageHash) {
        return [UPDATE_CHECK, deploymentKey, appVersion, label, packageHash].join(':');
    }
    clearUpdateCheckCache(deploymentKey, appVersion, label, packageHash, logger) {
        logger.info('clear cache Deployments key', {
            key: deploymentKey,
        });
        const redisCacheKey = this.getUpdateCheckCacheKey(deploymentKey, appVersion, label, packageHash);
        return connections_1.redisClient.keys(redisCacheKey).then((data) => {
            if (lodash_1.default.isArray(data)) {
                return Promise.all(data.map((key) => {
                    return connections_1.redisClient.del(key);
                }));
            }
            return null;
        });
    }
    updateCheckFromCache(deploymentKey, appVersion, label, packageHash, clientUniqueId, logger) {
        if (!config_1.config.common.updateCheckCache) {
            return this.updateCheck(deploymentKey, appVersion, label, packageHash, clientUniqueId, logger);
        }
        const redisCacheKey = this.getUpdateCheckCacheKey(deploymentKey, appVersion, label, packageHash);
        return connections_1.redisClient.get(redisCacheKey).then((data) => {
            if (data) {
                try {
                    logger.debug('updateCheckFromCache read from cache');
                    const obj = JSON.parse(data);
                    return obj;
                }
                catch (e) {
                    // do nothing
                }
            }
            return this.updateCheck(deploymentKey, appVersion, label, packageHash, clientUniqueId, logger).then((rs) => {
                try {
                    logger.debug('updateCheckFromCache read from db');
                    const strRs = JSON.stringify(rs);
                    connections_1.redisClient.setEx(redisCacheKey, EXPIRED, strRs);
                }
                catch (e) {
                    // do nothing
                }
                return rs;
            });
        });
    }
    getChosenManCacheKey(packageId, rollout, clientUniqueId) {
        return [CHOSEN_MAN, packageId, rollout, clientUniqueId].join(':');
    }
    random(rollout) {
        const r = Math.ceil(Math.random() * 10000);
        if (r < rollout * 100) {
            return Promise.resolve(true);
        }
        return Promise.resolve(false);
    }
    chosenMan(packageId, rollout, clientUniqueId) {
        if (rollout >= 100) {
            return Promise.resolve(true);
        }
        const rolloutClientUniqueIdCache = lodash_1.default.get(config_1.config, 'common.rolloutClientUniqueIdCache', false);
        if (rolloutClientUniqueIdCache === false) {
            return this.random(rollout);
        }
        const redisCacheKey = this.getChosenManCacheKey(packageId, rollout, clientUniqueId);
        return connections_1.redisClient.get(redisCacheKey).then((data) => {
            if (data === '1') {
                return true;
            }
            if (data === '2') {
                return false;
            }
            return this.random(rollout).then((r) => {
                return connections_1.redisClient
                    .setEx(redisCacheKey, 60 * 60 * 24 * 7, r ? '1' : '2')
                    .then(() => {
                    return r;
                });
            });
        });
    }
    // eslint-disable-next-line max-lines-per-function
    updateCheck(deploymentKey, appVersion, label, packageHash, clientUniqueId, logger) {
        const rs = {
            packageId: 0,
            downloadURL: '',
            downloadUrl: '',
            description: '',
            isAvailable: false,
            isDisabled: true,
            isMandatory: false,
            appVersion,
            targetBinaryRange: '',
            packageHash: '',
            label: '',
            packageSize: 0,
            updateAppVersion: false,
            shouldRunBinaryVersion: false,
            rollout: 100,
        };
        if (lodash_1.default.isEmpty(deploymentKey) || lodash_1.default.isEmpty(appVersion)) {
            return Promise.reject(new app_error_1.AppError('please input deploymentKey and appVersion'));
        }
        return deployments_1.Deployments.findOne({ where: { deployment_key: deploymentKey } })
            .then((dep) => {
            if (lodash_1.default.isEmpty(dep)) {
                throw new app_error_1.AppError('Not found deployment, check deployment key is right.');
            }
            const version = (0, common_1.parseVersion)(appVersion);
            return deployments_versions_1.DeploymentsVersions.findAll({
                where: {
                    deployment_id: dep.id,
                    min_version: { [sequelize_1.Op.lte]: version },
                    max_version: { [sequelize_1.Op.gt]: version },
                },
            }).then((deploymentsVersionsMore) => {
                const item = lodash_1.default.last(lodash_1.default.sortBy(deploymentsVersionsMore, 'created_at'));
                logger.debug({
                    item,
                });
                return item;
            });
        })
            .then((deploymentsVersions) => {
            const packageId = lodash_1.default.get(deploymentsVersions, 'current_package_id', 0);
            if (lodash_1.default.eq(packageId, 0)) {
                return undefined;
            }
            return packages_1.Packages.findByPk(packageId)
                .then((packages) => {
                if (packages &&
                    lodash_1.default.eq(packages.deployment_id, deploymentsVersions.deployment_id) &&
                    !lodash_1.default.eq(packages.package_hash, packageHash)) {
                    rs.packageId = packageId;
                    rs.targetBinaryRange = deploymentsVersions.app_version;
                    rs.downloadURL = (0, common_1.getBlobDownloadUrl)(packages.blob_url);
                    rs.downloadUrl = rs.downloadURL;
                    rs.description = lodash_1.default.get(packages, 'description', '');
                    rs.isAvailable = !lodash_1.default.eq(packages.is_disabled, 1);
                    rs.isDisabled = !!lodash_1.default.eq(packages.is_disabled, 1);
                    rs.isMandatory = !!lodash_1.default.eq(packages.is_mandatory, 1);
                    rs.appVersion = appVersion;
                    rs.packageHash = lodash_1.default.get(packages, 'package_hash', '');
                    rs.label = lodash_1.default.get(packages, 'label', '');
                    rs.packageSize = lodash_1.default.get(packages, 'size', 0);
                    rs.rollout = lodash_1.default.get(packages, 'rollout', 100);
                }
                return packages;
            })
                .then((packages) => {
                // 尝试增量更新
                if (packageHash &&
                    !lodash_1.default.isEmpty(packages) &&
                    !lodash_1.default.eq(lodash_1.default.get(packages, 'package_hash', ''), packageHash)) {
                    return packages_diff_1.PackagesDiff.findOne({
                        where: {
                            package_id: packages.id,
                            diff_against_package_hash: packageHash,
                        },
                    }).then((diffPackage) => {
                        if (!lodash_1.default.isEmpty(diffPackage)) {
                            rs.downloadURL = (0, common_1.getBlobDownloadUrl)(lodash_1.default.get(diffPackage, 'diff_blob_url'));
                            rs.downloadUrl = (0, common_1.getBlobDownloadUrl)(lodash_1.default.get(diffPackage, 'diff_blob_url'));
                            rs.packageSize = lodash_1.default.get(diffPackage, 'diff_size', 0);
                        }
                    });
                }
                return undefined;
            });
        })
            .then(() => {
            return rs;
        });
    }
    getPackagesInfo(deploymentKey, label) {
        if (lodash_1.default.isEmpty(deploymentKey) || lodash_1.default.isEmpty(label)) {
            return Promise.reject(new app_error_1.AppError('please input deploymentKey and label'));
        }
        return deployments_1.Deployments.findOne({ where: { deployment_key: deploymentKey } })
            .then((dep) => {
            if (lodash_1.default.isEmpty(dep)) {
                throw new app_error_1.AppError('does not found deployment');
            }
            return packages_1.Packages.findOne({ where: { deployment_id: dep.id, label } });
        })
            .then((packages) => {
            if (lodash_1.default.isEmpty(packages)) {
                throw new app_error_1.AppError('does not found packages');
            }
            return packages;
        });
    }
    reportStatusDownload(deploymentKey, label, clientUniqueId) {
        return this.getPackagesInfo(deploymentKey, label).then((packages) => {
            return Promise.all([
                packages_metrics_1.PackagesMetrics.findOne({ where: { package_id: packages.id } }).then((metrics) => {
                    if (metrics) {
                        return metrics.increment('downloaded');
                    }
                    return undefined;
                }),
                log_report_download_1.LogReportDownload.create({
                    package_id: packages.id,
                    client_unique_id: clientUniqueId,
                }),
            ]);
        });
    }
    reportStatusDeploy(deploymentKey, label, clientUniqueId, others) {
        return this.getPackagesInfo(deploymentKey, label).then((packages) => {
            const statusText = lodash_1.default.get(others, 'status');
            let status = 0;
            if (lodash_1.default.eq(statusText, 'DeploymentSucceeded')) {
                status = const_1.DEPLOYMENT_SUCCEEDED;
            }
            else if (lodash_1.default.eq(statusText, 'DeploymentFailed')) {
                status = const_1.DEPLOYMENT_FAILED;
            }
            const packageId = packages.id;
            const previousDeploymentKey = lodash_1.default.get(others, 'previousDeploymentKey');
            const previousLabel = lodash_1.default.get(others, 'previousLabelOrAppVersion');
            if (status > 0) {
                return Promise.all([
                    log_report_deploy_1.LogReportDeploy.create({
                        package_id: packageId,
                        client_unique_id: clientUniqueId,
                        previous_label: previousLabel,
                        previous_deployment_key: previousDeploymentKey,
                        status,
                    }),
                    packages_metrics_1.PackagesMetrics.findOne({ where: { package_id: packageId } }).then((metrics) => {
                        if (lodash_1.default.isEmpty(metrics)) {
                            return undefined;
                        }
                        if (lodash_1.default.eq(status, const_1.DEPLOYMENT_SUCCEEDED)) {
                            return metrics.increment(['installed', 'active'], { by: 1 });
                        }
                        return metrics.increment(['installed', 'failed'], { by: 1 });
                    }),
                ]).then(() => {
                    if (previousDeploymentKey && previousLabel) {
                        return deployments_1.Deployments.findOne({
                            where: { deployment_key: previousDeploymentKey },
                        })
                            .then((dep) => {
                            if (lodash_1.default.isEmpty(dep)) {
                                return undefined;
                            }
                            return packages_1.Packages.findOne({
                                where: { deployment_id: dep.id, label: previousLabel },
                            }).then((p) => {
                                if (lodash_1.default.isEmpty(p)) {
                                    return undefined;
                                }
                                return packages_metrics_1.PackagesMetrics.findOne({
                                    where: { package_id: p.id },
                                });
                            });
                        })
                            .then((metrics) => {
                            if (metrics) {
                                return metrics.decrement('active');
                            }
                            return undefined;
                        });
                    }
                    return undefined;
                });
            }
            return undefined;
        });
    }
}
exports.clientManager = new ClientManager();
