"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deploymentsManager = void 0;
const lodash_1 = __importDefault(require("lodash"));
const moment_1 = __importDefault(require("moment"));
const deployments_1 = require("../../models/deployments");
const deployments_history_1 = require("../../models/deployments_history");
const deployments_versions_1 = require("../../models/deployments_versions");
const packages_1 = require("../../models/packages");
const packages_diff_1 = require("../../models/packages_diff");
const packages_metrics_1 = require("../../models/packages_metrics");
const users_1 = require("../../models/users");
const app_error_1 = require("../app-error");
const common_1 = require("../utils/common");
const connections_1 = require("../utils/connections");
const security_1 = require("../utils/security");
class DeploymentsManager {
    getAllPackageIdsByDeploymentsId(deploymentsId) {
        return packages_1.Packages.findAll({ where: { deployment_id: deploymentsId } });
    }
    existDeloymentName(appId, name) {
        return deployments_1.Deployments.findOne({
            where: { appid: appId, name },
        }).then((data) => {
            if (!lodash_1.default.isEmpty(data)) {
                throw new app_error_1.AppError(`${name} name does Exist!`);
            }
            else {
                return data;
            }
        });
    }
    addDeloyment(name, appId, uid) {
        return users_1.Users.findByPk(uid).then((user) => {
            if (lodash_1.default.isEmpty(user)) {
                throw new app_error_1.AppError("can't find user");
            }
            return this.existDeloymentName(appId, name).then(() => {
                const { identical } = user;
                const deploymentKey = (0, security_1.randToken)(28) + identical;
                return deployments_1.Deployments.create({
                    appid: appId,
                    name,
                    deployment_key: deploymentKey,
                    last_deployment_version_id: 0,
                    label_id: 0,
                });
            });
        });
    }
    renameDeloymentByName(deploymentName, appId, newName) {
        return this.existDeloymentName(appId, newName).then(() => {
            return deployments_1.Deployments.update({ name: newName }, { where: { name: deploymentName, appid: appId } }).then(([affectedCount]) => {
                if (lodash_1.default.gt(affectedCount, 0)) {
                    return { name: newName };
                }
                throw new app_error_1.AppError(`does not find the deployment "${deploymentName}"`);
            });
        });
    }
    deleteDeloymentByName(deploymentName, appId) {
        return deployments_1.Deployments.destroy({
            where: { name: deploymentName, appid: appId },
        }).then((rowNum) => {
            if (lodash_1.default.gt(rowNum, 0)) {
                return { name: `${deploymentName}` };
            }
            throw new app_error_1.AppError(`does not find the deployment "${deploymentName}"`);
        });
    }
    findDeloymentByName(deploymentName, appId, logger) {
        logger.debug('findDeloymentByName', {
            name: deploymentName,
            appId,
        });
        return deployments_1.Deployments.findOne({
            where: { name: deploymentName, appid: appId },
        });
    }
    findPackagesAndOtherInfos(packageId) {
        return packages_1.Packages.findOne({
            where: { id: packageId },
        })
            .then((packageInfo) => {
            if (!packageInfo) {
                return null;
            }
            return Promise.all([
                Promise.resolve(packageInfo),
                packages_diff_1.PackagesDiff.findAll({
                    where: { package_id: packageId },
                }).then((diffs) => {
                    if (diffs.length > 0) {
                        return lodash_1.default.reduce(diffs, (result, v) => {
                            result[v.diff_against_package_hash] = {
                                size: v.diff_size,
                                url: (0, common_1.getBlobDownloadUrl)(v.diff_blob_url),
                            };
                            return result;
                        }, {});
                    }
                    return null;
                }),
                users_1.Users.findOne({
                    where: { id: packageInfo.released_by },
                }),
                deployments_versions_1.DeploymentsVersions.findByPk(packageInfo.deployment_version_id),
            ]);
        })
            .then(([packageInfo, packageDiffMap, userInfo, deploymentsVersions]) => {
            return {
                packageInfo,
                packageDiffMap,
                userInfo,
                deploymentsVersions,
            };
        });
    }
    findDeloymentsPackages(deploymentsVersionsId) {
        return deployments_versions_1.DeploymentsVersions.findOne({
            where: { id: deploymentsVersionsId },
        }).then((deploymentsVersionsInfo) => {
            if (deploymentsVersionsInfo) {
                return this.findPackagesAndOtherInfos(deploymentsVersionsInfo.current_package_id);
            }
            return null;
        });
    }
    formatPackage(packageVersion) {
        if (!packageVersion) {
            return null;
        }
        return {
            description: packageVersion.packageInfo.description,
            isDisabled: false,
            isMandatory: packageVersion.packageInfo.is_mandatory === 1,
            rollout: 100,
            appVersion: packageVersion.deploymentsVersions.app_version,
            packageHash: packageVersion.packageInfo.package_hash,
            blobUrl: (0, common_1.getBlobDownloadUrl)(packageVersion.packageInfo.blob_url),
            size: packageVersion.packageInfo.size,
            manifestBlobUrl: (0, common_1.getBlobDownloadUrl)(packageVersion.packageInfo.manifest_blob_url),
            diffPackageMap: packageVersion.packageDiffMap,
            releaseMethod: packageVersion.packageInfo.release_method,
            uploadTime: (0, moment_1.default)(packageVersion.packageInfo.updated_at).valueOf(),
            originalLabel: packageVersion.packageInfo.original_label,
            originalDeployment: packageVersion.packageInfo.original_deployment,
            label: packageVersion.packageInfo.label,
            releasedBy: packageVersion.userInfo.email,
        };
    }
    listDeloyments(appId) {
        return deployments_1.Deployments.findAll({ where: { appid: appId } }).then((deploymentsInfos) => {
            if (lodash_1.default.isEmpty(deploymentsInfos)) {
                return [];
            }
            return Promise.all(deploymentsInfos.map((v) => {
                return this.listDeloyment(v);
            }));
        });
    }
    listDeloyment(deploymentInfo) {
        return this.findDeloymentsPackages([deploymentInfo.last_deployment_version_id])
            .then(this.formatPackage)
            .then((packageInfo) => {
            return {
                createdTime: (0, moment_1.default)(deploymentInfo.created_at).valueOf(),
                id: `${deploymentInfo.id}`,
                key: deploymentInfo.deployment_key,
                name: deploymentInfo.name,
                package: packageInfo,
            };
        });
    }
    getDeploymentHistory(deploymentId) {
        return deployments_history_1.DeploymentsHistory.findAll({
            where: { deployment_id: deploymentId },
            order: [['id', 'desc']],
            limit: 15,
        })
            .then((history) => {
            return lodash_1.default.map(history, (v) => {
                return v.package_id;
            });
        })
            .then((packageIds) => {
            return Promise.all(packageIds.map((v) => {
                return this.findPackagesAndOtherInfos(v).then(this.formatPackage);
            }));
        });
    }
    deleteDeploymentHistory(deploymentId) {
        return connections_1.sequelize.transaction((t) => {
            return Promise.all([
                deployments_1.Deployments.update({ last_deployment_version_id: 0, label_id: 0 }, { where: { id: deploymentId }, transaction: t }),
                deployments_history_1.DeploymentsHistory.findAll({
                    where: { deployment_id: deploymentId },
                    order: [['id', 'desc']],
                    limit: 1000,
                }).then((rs) => {
                    return Promise.all(rs.map((v) => {
                        return v.destroy({ transaction: t });
                    }));
                }),
                deployments_versions_1.DeploymentsVersions.findAll({
                    where: { deployment_id: deploymentId },
                    order: [['id', 'desc']],
                    limit: 1000,
                }).then((rs) => {
                    return Promise.all(rs.map((v) => {
                        return v.destroy({ transaction: t });
                    }));
                }),
                packages_1.Packages.findAll({
                    where: { deployment_id: deploymentId },
                    order: [['id', 'desc']],
                    limit: 1000,
                }).then((rs) => {
                    return Promise.all(rs.map((v) => {
                        return v.destroy({ transaction: t }).then(() => {
                            return Promise.all([
                                packages_metrics_1.PackagesMetrics.destroy({
                                    where: { package_id: v.get('id') },
                                    transaction: t,
                                }),
                                packages_diff_1.PackagesDiff.destroy({
                                    where: { package_id: v.get('id') },
                                    transaction: t,
                                }),
                            ]);
                        });
                    }));
                }),
            ]);
        });
    }
}
exports.deploymentsManager = new DeploymentsManager();
